!   D01AQF Example Program Text
!   Mark 23 Release. NAG Copyright 2011.

    MODULE d01aqfe_mod

!      D01AQF Example Program Module:
!             Parameters and User-defined Routines

!      .. Use Statements ..
       USE nag_library, ONLY : nag_wp
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER                  :: lw = 800, nout = 6
       INTEGER, PARAMETER                  :: liw = lw/4
    CONTAINS
       FUNCTION g(x)

!         .. Implicit None Statement ..
          IMPLICIT NONE
!         .. Function Return Value ..
          REAL (KIND=nag_wp)                  :: g
!         .. Scalar Arguments ..
          REAL (KIND=nag_wp), INTENT (IN)     :: x
!         .. Local Scalars ..
          REAL (KIND=nag_wp)                  :: aa
!         .. Executable Statements ..
          aa = 0.01E0_nag_wp
          g = 1.0E0_nag_wp/(x**2+aa**2)

          RETURN

       END FUNCTION g
    END MODULE d01aqfe_mod
    PROGRAM d01aqfe

!      D01AQF Example Main Program

!      .. Use Statements ..
       USE nag_library, ONLY : d01aqf, nag_wp
       USE d01aqfe_mod, ONLY : g, liw, lw, nout
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Local Scalars ..
       REAL (KIND=nag_wp)                  :: a, abserr, b, c, epsabs, epsrel, &
                                              result
       INTEGER                             :: ifail
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE     :: w(:)
       INTEGER, ALLOCATABLE                :: iw(:)
!      .. Executable Statements ..
       WRITE (nout,*) 'D01AQF Example Program Results'

       ALLOCATE (w(lw),iw(liw))

       epsabs = 0.0E0_nag_wp
       epsrel = 1.0E-04_nag_wp
       a = -1.0E0_nag_wp
       b = 1.0E0_nag_wp
       c = 0.5E0_nag_wp

       ifail = -1
       CALL d01aqf(g,a,b,c,epsabs,epsrel,result,abserr,w,lw,iw,liw,ifail)

       IF (ifail>=0) THEN
          WRITE (nout,*)
          WRITE (nout,99999) 'A     ', 'lower limit of integration', a
          WRITE (nout,99999) 'B     ', 'upper limit of integration', b
          WRITE (nout,99998) 'EPSABS', 'absolute accuracy requested', epsabs
          WRITE (nout,99998) 'EPSREL', 'relative accuracy requested', epsrel
          WRITE (nout,99998) 'C     ', 'weight function parameter', c
       END IF

       IF (ifail>=0 .AND. ifail<=3) THEN
          WRITE (nout,*)
          WRITE (nout,99997) 'RESULT', 'approximation to the integral', result
          WRITE (nout,99998) 'ABSERR', 'estimate of the absolute error', &
             abserr
          WRITE (nout,99996) 'IW(1) ', 'number of subintervals used', iw(1)
       END IF

99999  FORMAT (1X,A6,' - ',A32,' = ',F10.4)
99998  FORMAT (1X,A6,' - ',A32,' = ',E9.2)
99997  FORMAT (1X,A6,' - ',A32,' = ',F9.2)
99996  FORMAT (1X,A6,' - ',A32,' = ',I4)
    END PROGRAM d01aqfe
