    PROGRAM f04bbfe

!      F04BBF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : f04bbf, nag_wp, x04caf, x04cef
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: errbnd, rcond
       INTEGER                         :: i, ierr, ifail, j, k, kl, ku, ldab,  &
                                          ldb, n, nrhs
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: ab(:,:), b(:,:)
       INTEGER, ALLOCATABLE            :: ipiv(:)
!      .. Intrinsic Functions ..
       INTRINSIC                          max, min
!      .. Executable Statements ..
       WRITE (nout,*) 'F04BBF Example Program Results'
       WRITE (nout,*)
       FLUSH (nout)
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, kl, ku, nrhs
       ldab = 2*kl + ku + 1
       ldb = n
       ALLOCATE (ab(ldab,n),b(ldb,nrhs),ipiv(n))
!      Read A and B from data file
       k = kl + ku + 1
       READ (nin,*) ((ab(k+i-j,j),j=max(i-kl,1),min(i+ku,n)),i=1,n)
       READ (nin,*) (b(i,1:nrhs),i=1,n)

!      Solve the equations AX = B for X

!      ifail: behaviour on error exit
!             =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
       ifail = 1
       CALL f04bbf(n,kl,ku,nrhs,ab,ldab,ipiv,b,ldb,rcond,errbnd,ifail)

       IF (ifail==0) THEN

!         Print solution, estimate of condition number and approximate
!         error bound

          ierr = 0
          CALL x04caf('General',' ',n,nrhs,b,ldb,'Solution',ierr)

          WRITE (nout,*)
          WRITE (nout,*) 'Estimate of condition number'
          WRITE (nout,99999) 1.0E0_nag_wp/rcond
          WRITE (nout,*)
          WRITE (nout,*) 'Estimate of error bound for computed solutions'
          WRITE (nout,99999) errbnd
       ELSE IF (ifail==n+1) THEN

!         Matrix A is numerically singular.  Print estimate of
!         reciprocal of condition number and solution

          WRITE (nout,*)
          WRITE (nout,*) 'Estimate of reciprocal of condition number'
          WRITE (nout,99999) rcond
          WRITE (nout,*)
          FLUSH (nout)

          ierr = 0
          CALL x04caf('General',' ',n,nrhs,b,ldb,'Solution',ierr)

       ELSE IF (ifail>0 .AND. ifail<=n) THEN

!         The upper triangular matrix U is exactly singular.  Print
!         details of factorization
          WRITE (nout,*)
          FLUSH (nout)

          ierr = 0
          CALL x04cef(n,n,kl,kl+ku,ab,ldab,'Details of factorization',ierr)

!         Print pivot indices
          WRITE (nout,*)
          WRITE (nout,*) 'Pivot indices'
          WRITE (nout,99998) ipiv(1:n)
       ELSE
          WRITE (nout,99997) ifail
       END IF

99999  FORMAT (6X,1P,E9.1)
99998  FORMAT ((1X,7I11))
99997  FORMAT (1X,' ** F04BBF returned with IFAIL = ',I5)
    END PROGRAM f04bbfe
