    PROGRAM f04bgfe

!      F04BGF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : f04bgf, nag_wp, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: errbnd, rcond
       INTEGER                         :: i, ierr, ifail, ldb, n, nrhs
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: b(:,:), d(:), e(:)
!      .. Executable Statements ..
       WRITE (nout,*) 'F04BGF Example Program Results'
       WRITE (nout,*)
       FLUSH (nout)
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, nrhs
       ldb = n
       ALLOCATE (b(ldb,nrhs),d(n),e(n-1))
!      Read A from data file
       READ (nin,*) d(1:n)
       READ (nin,*) e(1:n-1)

!      Read B from data file
       READ (nin,*) (b(i,1:nrhs),i=1,n)

!      Solve the equations AX = B for X

!      ifail: behaviour on error exit
!             =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
       ifail = 1
       CALL f04bgf(n,nrhs,d,e,b,ldb,rcond,errbnd,ifail)

       IF (ifail==0) THEN
!         Print solution, estimate of condition number and approximate
!         error bound

          ierr = 0
          CALL x04caf('General',' ',n,nrhs,b,ldb,'Solution',ierr)

          WRITE (nout,*)
          WRITE (nout,*) 'Estimate of condition number'
          WRITE (nout,99999) 1.0E0_nag_wp/rcond
          WRITE (nout,*)
          WRITE (nout,*) 'Estimate of error bound for computed solutions'
          WRITE (nout,99999) errbnd
       ELSE IF (ifail==n+1) THEN
!         Matrix A is numerically singular.  Print estimate of
!         reciprocal of condition number and solution
          WRITE (nout,*)
          WRITE (nout,*) 'Estimate of reciprocal of condition number'
          WRITE (nout,99999) rcond
          WRITE (nout,*)
          FLUSH (nout)

          ierr = 0
          CALL x04caf('General',' ',n,nrhs,b,ldb,'Solution',ierr)

       ELSE IF (ifail>0 .AND. ifail<=n) THEN
          WRITE (nout,99998) 'The leading minor of order ', ifail, &
             ' is not positive definite'
       ELSE
          WRITE (nout,99997) ifail
       END IF

99999  FORMAT (6X,1P,E9.1)
99998  FORMAT (1X,A,I3,A)
99997  FORMAT (1X,' ** F04BGF returned with IFAIL = ',I5)
    END PROGRAM f04bgfe
