    PROGRAM f07bvfe

!      F07BVF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : nag_wp, x04dbf, zgbrfs, zgbtrf, zgbtrs
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       COMPLEX (KIND=nag_wp), PARAMETER :: zero = (0.0_nag_wp,0.0_nag_wp)
       INTEGER, PARAMETER              :: nin = 5, nout = 6
       CHARACTER (1), PARAMETER        :: trans = 'N'
!      .. Local Scalars ..
       INTEGER                         :: i, ifail, info, j, k, kl, ku, ldab,  &
                                          ldafb, ldb, ldx, n, nrhs
!      .. Local Arrays ..
       COMPLEX (KIND=nag_wp), ALLOCATABLE :: ab(:,:), afb(:,:), b(:,:),        &
                                             work(:), x(:,:)
       REAL (KIND=nag_wp), ALLOCATABLE :: berr(:), ferr(:), rwork(:)
       INTEGER, ALLOCATABLE            :: ipiv(:)
       CHARACTER (1)                   :: clabs(1), rlabs(1)
!      .. Intrinsic Functions ..
       INTRINSIC                          max, min
!      .. Executable Statements ..
       WRITE (nout,*) 'F07BVF Example Program Results'
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, nrhs, kl, ku
       ldb = n
       ldx = n
       ldab = kl + ku + 1
       ldafb = 2*kl + ku + 1
       ALLOCATE (ab(ldab,n),afb(ldafb,n),b(ldb,nrhs),work(2*n),x(ldx,n), &
          berr(nrhs),ferr(nrhs),rwork(n),ipiv(n))

!      Set A to zero to avoid referencing uninitialized elements

       ab(1:kl+ku+1,1:n) = zero

!      Read A and B from data file, and copy A to AFB and B to X

       k = ku + 1
       READ (nin,*) ((ab(k+i-j,j),j=max(i-kl,1),min(i+ku,n)),i=1,n)
       READ (nin,*) (b(i,1:nrhs),i=1,n)

       afb(kl+1:2*kl+ku+1,1:n) = ab(1:kl+ku+1,1:n)
       x(1:n,1:nrhs) = b(1:n,1:nrhs)

!      Factorize A in the array AFB
!      The NAG name equivalent of zgbtrf is f07brf
       CALL zgbtrf(n,n,kl,ku,afb,ldafb,ipiv,info)

       WRITE (nout,*)
       FLUSH (nout)
       IF (info==0) THEN

!         Compute solution in the array X
!         The NAG name equivalent of zgbtrs is f07bsf
          CALL zgbtrs(trans,n,kl,ku,nrhs,afb,ldafb,ipiv,x,ldx,info)

!         Improve solution, and compute backward errors and
!         estimated bounds on the forward errors
!         The NAG name equivalent of zgbrfs is f07bvf
          CALL zgbrfs(trans,n,kl,ku,nrhs,ab,ldab,afb,ldafb,ipiv,b,ldb,x,ldx, &
             ferr,berr,work,rwork,info)

!         Print solution

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04dbf('General',' ',n,nrhs,x,ldx,'Bracketed','F7.4', &
             'Solution(s)','Integer',rlabs,'Integer',clabs,80,0,ifail)

          WRITE (nout,*)
          WRITE (nout,*) 'Backward errors (machine-dependent)'
          WRITE (nout,99999) berr(1:nrhs)
          WRITE (nout,*) 'Estimated forward error bounds (machine-dependent)'
          WRITE (nout,99999) ferr(1:nrhs)
       ELSE
          WRITE (nout,*) 'The factor U is singular'
       END IF

99999  FORMAT ((5X,1P,4(E11.1,7X)))
    END PROGRAM f07bvfe
