    PROGRAM f07chfe

!      F07CHF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dgtrfs, dgttrf, dgttrs, nag_wp, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       INTEGER                         :: i, ifail, info, ldb, ldx, n, nrhs
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: b(:,:), berr(:), d(:), df(:), dl(:), &
                                          dlf(:), du(:), du2(:), duf(:),       &
                                          ferr(:), work(:), x(:,:)
       INTEGER, ALLOCATABLE            :: ipiv(:), iwork(:)
!      .. Executable Statements ..
       WRITE (nout,*) 'F07CHF Example Program Results'
       WRITE (nout,*)
       FLUSH (nout)
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, nrhs
       ldb = n
       ldx = n
       ALLOCATE (b(ldb,nrhs),berr(nrhs),d(n),df(n),dl(n-1),dlf(n-1),du(n-1), &
          du2(n-2),duf(n-1),ferr(nrhs),work(3*n),x(ldx,nrhs),ipiv(n),iwork(n))

!      Read the tridiagonal matrix A from data file

       READ (nin,*) du(1:n-1)
       READ (nin,*) d(1:n)
       READ (nin,*) dl(1:n-1)

!      Read the right hand matrix B

       READ (nin,*) (b(i,1:nrhs),i=1,n)

!      Copy A into DUF, DF and DLF, and copy B into X

       duf(1:n-1) = du(1:n-1)
       df(1:n) = d(1:n)
       dlf(1:n-1) = dl(1:n-1)
       x(1:n,1:nrhs) = b(1:n,1:nrhs)

!      Factorize the copy of the tridiagonal matrix A
!      The NAG name equivalent of dgttrf is f07cdf
       CALL dgttrf(n,dlf,df,duf,du2,ipiv,info)

       IF (info==0) THEN

!         Solve the equations AX = B
!         The NAG name equivalent of dgttrs is f07cef
          CALL dgttrs('No transpose',n,nrhs,dlf,df,duf,du2,ipiv,x,ldx,info)

!         Improve the solution and compute error estimates
!         The NAG name equivalent of dgtrfs is f07chf
          CALL dgtrfs('No transpose',n,nrhs,dl,d,du,dlf,df,duf,du2,ipiv,b,ldb, &
             x,ldx,ferr,berr,work,iwork,info)

!         Print the solution and the forward and backward error
!         estimates

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04caf('General',' ',n,nrhs,x,ldx,'Solution(s)',ifail)

          WRITE (nout,*)
          WRITE (nout,*) 'Backward errors (machine-dependent)'
          WRITE (nout,99999) berr(1:nrhs)
          WRITE (nout,*)
          WRITE (nout,*) 'Estimated forward error bounds (machine-dependent)'
          WRITE (nout,99999) ferr(1:nrhs)
       ELSE
          WRITE (nout,99998) 'The (', info, ',', info, ')', &
             ' element of the factor U is zero'
       END IF

99999  FORMAT ((3X,1P,7E11.1))
99998  FORMAT (1X,A,I3,A,I3,A,A)
    END PROGRAM f07chfe
