    PROGRAM f07cpfe

!      F07CPF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : nag_wp, x04dbf, zgtsvx
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: rcond
       INTEGER                         :: i, ifail, info, ldb, ldx, n, nrhs
!      .. Local Arrays ..
       COMPLEX (KIND=nag_wp), ALLOCATABLE :: b(:,:), d(:), df(:), dl(:),       &
                                             dlf(:), du(:), du2(:), duf(:),    &
                                             work(:), x(:,:)
       REAL (KIND=nag_wp), ALLOCATABLE :: berr(:), ferr(:), rwork(:)
       INTEGER, ALLOCATABLE            :: ipiv(:)
       CHARACTER (1)                   :: clabs(1), rlabs(1)
!      .. Executable Statements ..
       WRITE (nout,*) 'F07CPF Example Program Results'
       WRITE (nout,*)
       FLUSH (nout)
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, nrhs
       ldb = n
       ldx = n
       ALLOCATE (b(ldb,nrhs),d(n),df(n),dl(n-1),dlf(n-1),du(n-1),du2(n-2), &
          duf(n-1),work(2*n),x(ldx,nrhs),berr(nrhs),ferr(nrhs),rwork(n), &
          ipiv(n))

!      Read the tridiagonal matrix A from data file

       READ (nin,*) du(1:n-1)
       READ (nin,*) d(1:n)
       READ (nin,*) dl(1:n-1)

!      Read the right hand matrix B

       READ (nin,*) (b(i,1:nrhs),i=1,n)

!      Solve the equations AX = B
!      The NAG name equivalent of zgtsvx is f07cpf
       CALL zgtsvx('No factors','No transpose',n,nrhs,dl,d,du,dlf,df,duf,du2, &
          ipiv,b,ldb,x,ldx,rcond,ferr,berr,work,rwork,info)

       IF ((info==0) .OR. (info==n+1)) THEN

!         Print solution, error bounds and condition number

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04dbf('General',' ',n,nrhs,x,ldx,'Bracketed','F7.4', &
             'Solution(s)','Integer',rlabs,'Integer',clabs,80,0,ifail)

          WRITE (nout,*)
          WRITE (nout,*) 'Backward errors (machine-dependent)'
          WRITE (nout,99999) berr(1:nrhs)
          WRITE (nout,*)
          WRITE (nout,*) 'Estimated forward error bounds (machine-dependent)'
          WRITE (nout,99999) ferr(1:nrhs)
          WRITE (nout,*)
          WRITE (nout,*) 'Estimate of reciprocal condition number'
          WRITE (nout,99999) rcond

          IF (info==n+1) THEN
             WRITE (nout,*)
             WRITE (nout,*) 'The matrix A is singular to working precision'
          END IF
       ELSE
          WRITE (nout,99998) 'The (', info, ',', info, ')', &
             ' element of the factor U is zero'
       END IF

99999  FORMAT ((3X,1P,7E11.1))
99998  FORMAT (1X,A,I3,A,I3,A,A)
    END PROGRAM f07cpfe
