    PROGRAM f07mhfe

!      F07MHF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dsyrfs, dsytrf, dsytrs, nag_wp, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       INTEGER                         :: i, ifail, info, lda, ldaf, ldb, ldx, &
                                          lwork, n, nrhs
       CHARACTER (1)                   :: uplo
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: a(:,:), af(:,:), b(:,:), berr(:),    &
                                          ferr(:), work(:), x(:,:)
       INTEGER, ALLOCATABLE            :: ipiv(:), iwork(:)
!      .. Executable Statements ..
       WRITE (nout,*) 'F07MHF Example Program Results'
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, nrhs
       lda = n
       ldaf = n
       ldb = n
       ldx = n
       lwork = 64*n
       ALLOCATE (a(lda,n),af(ldaf,n),b(ldb,nrhs),berr(nrhs),ferr(nrhs), &
          work(lwork),x(ldx,n),ipiv(n),iwork(n))

!      Read A and B from data file, and copy A to AF and B to X

       READ (nin,*) uplo
       IF (uplo=='U') THEN
          DO i = 1, n
             READ (nin,*) a(i,i:n)
             af(i,i:n) = a(i,i:n)
          END DO
       ELSE IF (uplo=='L') THEN
          DO i = 1, n
             READ (nin,*) a(i,1:i)
             af(i,1:i) = a(i,1:i)
          END DO
       END IF
       READ (nin,*) (b(i,1:nrhs),i=1,n)

       x(1:n,1:nrhs) = b(1:n,1:nrhs)

!      Factorize A in the array AF
!      The NAG name equivalent of dsytrf is f07mdf
       CALL dsytrf(uplo,n,af,ldaf,ipiv,work,lwork,info)

       WRITE (nout,*)
       FLUSH (nout)
       IF (info==0) THEN

!         Compute solution in the array X
!         The NAG name equivalent of dsytrs is f07mef
          CALL dsytrs(uplo,n,nrhs,af,ldaf,ipiv,x,ldx,info)

!         Improve solution, and compute backward errors and
!         estimated bounds on the forward errors
!         The NAG name equivalent of dsyrfs is f07mhf
          CALL dsyrfs(uplo,n,nrhs,a,lda,af,ldaf,ipiv,b,ldb,x,ldx,ferr,berr, &
             work,iwork,info)

!         Print solution

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04caf('General',' ',n,nrhs,x,ldx,'Solution(s)',ifail)

          WRITE (nout,*)
          WRITE (nout,*) 'Backward errors (machine-dependent)'
          WRITE (nout,99999) berr(1:nrhs)
          WRITE (nout,*) 'Estimated forward error bounds (machine-dependent)'
          WRITE (nout,99999) ferr(1:nrhs)
       ELSE
          WRITE (nout,*) 'The factor D is singular'
       END IF

99999  FORMAT ((3X,1P,7E11.1))
    END PROGRAM f07mhfe
