    PROGRAM f07nvfe

!      F07NVF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : f06tff, nag_wp, x04dbf, zsyrfs, zsytrf, zsytrs
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       INTEGER                         :: i, ifail, info, lda, ldaf, ldb, ldx, &
                                          lwork, n, nrhs
       CHARACTER (1)                   :: uplo
!      .. Local Arrays ..
       COMPLEX (KIND=nag_wp), ALLOCATABLE :: a(:,:), af(:,:), b(:,:), work(:), &
                                             x(:,:)
       REAL (KIND=nag_wp), ALLOCATABLE :: berr(:), ferr(:), rwork(:)
       INTEGER, ALLOCATABLE            :: ipiv(:)
       CHARACTER (1)                   :: clabs(1), rlabs(1)
!      .. Executable Statements ..
       WRITE (nout,*) 'F07NVF Example Program Results'
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, nrhs
       lda = n
       ldaf = n
       ldb = n
       ldx = n
       lwork = 64*n
       ALLOCATE (a(lda,n),af(ldaf,n),b(ldb,nrhs),work(lwork),x(ldx,n), &
          berr(nrhs),ferr(nrhs),rwork(n),ipiv(n))

!      Read A and B from data file, and copy A to AF and B to X

       READ (nin,*) uplo
       IF (uplo=='U') THEN
          READ (nin,*) (a(i,i:n),i=1,n)
       ELSE IF (uplo=='L') THEN
          READ (nin,*) (a(i,1:i),i=1,n)
       END IF
       READ (nin,*) (b(i,1:nrhs),i=1,n)
       CALL f06tff(uplo,n,n,a,lda,af,ldaf)
       x(1:n,1:nrhs) = b(1:n,1:nrhs)

!      Factorize A in the array AF
!      The NAG name equivalent of zsytrf is f07nrf
       CALL zsytrf(uplo,n,af,ldaf,ipiv,work,lwork,info)

       WRITE (nout,*)
       FLUSH (nout)
       IF (info==0) THEN

!         Compute solution in the array X
!         The NAG name equivalent of zsytrs is f07nsf
          CALL zsytrs(uplo,n,nrhs,af,ldaf,ipiv,x,ldx,info)

!         Improve solution, and compute backward errors and
!         estimated bounds on the forward errors

!         The NAG name equivalent of zsyrfs is f07nvf
          CALL zsyrfs(uplo,n,nrhs,a,lda,af,ldaf,ipiv,b,ldb,x,ldx,ferr,berr, &
             work,rwork,info)

!         Print solution

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04dbf('General',' ',n,nrhs,x,ldx,'Bracketed','F7.4', &
             'Solution(s)','Integer',rlabs,'Integer',clabs,80,0,ifail)

          WRITE (nout,*)
          WRITE (nout,*) 'Backward errors (machine-dependent)'
          WRITE (nout,99999) berr(1:nrhs)
          WRITE (nout,*) 'Estimated forward error bounds (machine-dependent)'
          WRITE (nout,99999) ferr(1:nrhs)
       ELSE
          WRITE (nout,*) 'The factor D is singular'
       END IF

99999  FORMAT ((5X,1P,4(E11.1,7X)))
    END PROGRAM f07nvfe
