    PROGRAM f07vhfe

!      F07VHF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dtbrfs, dtbtrs, nag_wp, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
       CHARACTER (1), PARAMETER        :: diag = 'N', trans = 'N'
!      .. Local Scalars ..
       INTEGER                         :: i, ifail, info, j, kd, ldab, ldb,    &
                                          ldx, n, nrhs
       CHARACTER (1)                   :: uplo
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: ab(:,:), b(:,:), berr(:), ferr(:),   &
                                          work(:), x(:,:)
       INTEGER, ALLOCATABLE            :: iwork(:)
!      .. Intrinsic Functions ..
       INTRINSIC                          max, min
!      .. Executable Statements ..
       WRITE (nout,*) 'F07VHF Example Program Results'
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n, kd, nrhs
       ldab = kd + 1
       ldb = n
       ldx = n
       ALLOCATE (ab(ldab,n),b(ldb,nrhs),berr(nrhs),ferr(nrhs),work(3*n), &
          x(ldx,n),iwork(n))

!      Read A and B from data file, and copy B to X

       READ (nin,*) uplo
       IF (uplo=='U') THEN
          DO i = 1, n
             READ (nin,*) (ab(kd+1+i-j,j),j=i,min(n,i+kd))
          END DO
       ELSE IF (uplo=='L') THEN
          DO i = 1, n
             READ (nin,*) (ab(1+i-j,j),j=max(1,i-kd),i)
          END DO
       END IF
       READ (nin,*) (b(i,1:nrhs),i=1,n)

       x(1:n,1:nrhs) = b(1:n,1:nrhs)

!      Compute solution in the array X
!      The NAG name equivalent of dtbtrs is f07vef
       CALL dtbtrs(uplo,trans,diag,n,kd,nrhs,ab,ldab,x,ldx,info)

!      Compute backward errors and estimated bounds on the
!      forward errors

!      The NAG name equivalent of dtbrfs is f07vhf
       CALL dtbrfs(uplo,trans,diag,n,kd,nrhs,ab,ldab,b,ldb,x,ldx,ferr,berr, &
          work,iwork,info)

!      Print solution

       WRITE (nout,*)
       FLUSH (nout)

!      ifail: behaviour on error exit
!             =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
       ifail = 0
       CALL x04caf('General',' ',n,nrhs,x,ldx,'Solution(s)',ifail)

       WRITE (nout,*)
       WRITE (nout,*) 'Backward errors (machine-dependent)'
       WRITE (nout,99999) berr(1:nrhs)
       WRITE (nout,*) 'Estimated forward error bounds (machine-dependent)'
       WRITE (nout,99999) ferr(1:nrhs)

99999  FORMAT ((3X,1P,7E11.1))
    END PROGRAM f07vhfe
