    PROGRAM f08fffe

!      F08FFF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dorgtr, dsteqr, dsytrd, f06qff, nag_wp, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       INTEGER                         :: i, ifail, info, lda, ldz, lwork, n
       CHARACTER (1)                   :: uplo
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: a(:,:), d(:), e(:), tau(:), work(:), &
                                          z(:,:)
!      .. Executable Statements ..
       WRITE (nout,*) 'F08FFF Example Program Results'
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n
       lda = n
       ldz = n
       lwork = 64*n
       ALLOCATE (a(lda,n),d(n),e(n),tau(n),work(lwork),z(ldz,n))

!      Read A from data file

       READ (nin,*) uplo
       IF (uplo=='U') THEN
          READ (nin,*) (a(i,i:n),i=1,n)
       ELSE IF (uplo=='L') THEN
          READ (nin,*) (a(i,1:i),i=1,n)
       END IF

!      Reduce A to tridiagonal form T = (Q**T)*A*Q
!      The NAG name equivalent of dsytrd is f08fef
       CALL dsytrd(uplo,n,a,lda,d,e,tau,work,lwork,info)

!      Copy A into Z
       CALL f06qff(uplo,n,n,a,lda,z,ldz)

!      Form Q explicitly, storing the result in Z
!      The NAG name equivalent of dorgtr is f08fff
       CALL dorgtr(uplo,n,z,ldz,tau,work,lwork,info)

!      Calculate all the eigenvalues and eigenvectors of A
!      The NAG name equivalent of dsteqr is f08jef
       CALL dsteqr('V',n,d,e,z,ldz,work,info)

       WRITE (nout,*)
       IF (info>0) THEN
          WRITE (nout,*) 'Failure to converge.'
       ELSE

!         Print eigenvalues and eigenvectors

          WRITE (nout,*) 'Eigenvalues'
          WRITE (nout,99999) d(1:n)
          WRITE (nout,*)
          FLUSH (nout)

!         Normalize the eigenvectors
          DO i = 1, n
             z(1:n,i) = z(1:n,i)/z(1,i)
          END DO

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04caf('General',' ',n,n,z,ldz,'Eigenvectors',ifail)

       END IF

99999  FORMAT (3X,(8F8.4))
    END PROGRAM f08fffe
