    PROGRAM f08jafe

!      F08JAF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : ddisna, dstev, nag_wp, x02ajf, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       REAL (KIND=nag_wp)              :: eerrbd, eps
       INTEGER                         :: i, ifail, info, ldz, n
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: d(:), e(:), rcondz(:), work(:),      &
                                          z(:,:), zerrbd(:)
!      .. Intrinsic Functions ..
       INTRINSIC                          abs, max
!      .. Executable Statements ..
       WRITE (nout,*) 'F08JAF Example Program Results'
       WRITE (nout,*)
!      Skip heading in data file
       READ (nin,*)
       READ (nin,*) n
       ldz = n
       ALLOCATE (d(n),e(n-1),rcondz(n),work(2*n-2),z(ldz,n),zerrbd(n))

!      Read the diagonal and off-diagonal elements of the matrix A
!      from data file

       READ (nin,*) d(1:n)
       READ (nin,*) e(1:n-1)

!      Solve the symmetric tridiagonal eigenvalue problem
!      The NAG name equivalent of dstev is f08jaf
       CALL dstev('Vectors',n,d,e,z,ldz,work,info)

       IF (info==0) THEN

!         Print solution

          WRITE (nout,*) 'Eigenvalues'
          WRITE (nout,99999) d(1:n)
          FLUSH (nout)

!         Normalize the eigenvectors
          DO i = 1, n
             z(1:n,i) = z(1:n,i)/z(1,i)
          END DO

!         ifail: behaviour on error exit
!                =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
          ifail = 0
          CALL x04caf('General',' ',n,n,z,ldz,'Eigenvectors',ifail)

!         Get the machine precision, EPS and compute the approximate
!         error bound for the computed eigenvalues.  Note that for
!         the 2-norm, max( abs(D(i)) ) = norm(A), and since the
!         eigenvalues are returned in ascending order
!         max( abs(D(i)) ) = max( abs(D(1)), abs(D(n)))

          eps = x02ajf()
          eerrbd = eps*max(abs(d(1)),abs(d(n)))

!         Call DDISNA (F08FLF) to estimate reciprocal condition
!         numbers for the eigenvectors
          CALL ddisna('Eigenvectors',n,n,d,rcondz,info)

!         Compute the error estimates for the eigenvectors

          DO i = 1, n
             zerrbd(i) = eerrbd/rcondz(i)
          END DO

!         Print the approximate error bounds for the eigenvalues
!         and vectors

          WRITE (nout,*)
          WRITE (nout,*) 'Error estimate for the eigenvalues'
          WRITE (nout,99998) eerrbd
          WRITE (nout,*)
          WRITE (nout,*) 'Error estimates for the eigenvectors'
          WRITE (nout,99998) zerrbd(1:n)
       ELSE
          WRITE (nout,99997) 'Failure in DSTEV. INFO =', info
       END IF

99999  FORMAT (3X,(8F8.4))
99998  FORMAT (4X,1P,6E11.1)
99997  FORMAT (1X,A,I4)
    END PROGRAM f08jafe
