    PROGRAM f08ykfe

!      F08YKF Example Program Text

!      Mark 23 Release. NAG Copyright 2011.

!      .. Use Statements ..
       USE nag_library, ONLY : dgeqrf, dggbak, dggbal, dgghrd, dhgeqz, dorgqr, &
                               dormqr, dtgevc, f06qff, f06qhf, nag_wp, x04caf
!      .. Implicit None Statement ..
       IMPLICIT NONE
!      .. Parameters ..
       REAL (KIND=nag_wp), PARAMETER   :: one = 1.0E0_nag_wp
       REAL (KIND=nag_wp), PARAMETER   :: zero = 0.0E0_nag_wp
       INTEGER, PARAMETER              :: nin = 5, nout = 6
!      .. Local Scalars ..
       INTEGER                         :: i, icols, ifail, ihi, ilo, info,     &
                                          irows, jwork, lda, ldb, ldvl, ldvr,  &
                                          lwork, m, n
       LOGICAL                         :: ileft, iright
       CHARACTER (1)                   :: compq, compz, howmny, job, side
!      .. Local Arrays ..
       REAL (KIND=nag_wp), ALLOCATABLE :: a(:,:), alphai(:), alphar(:),        &
                                          b(:,:), beta(:), lscale(:),          &
                                          rscale(:), tau(:), vl(:,:), vr(:,:), &
                                          work(:)
       LOGICAL, ALLOCATABLE            :: select(:)
!      .. Intrinsic Functions ..
       INTRINSIC                          nint
!      .. Executable Statements ..
       WRITE (nout,*) 'F08YKF Example Program Results'
       FLUSH (nout)

!      ileft  is TRUE if left  eigenvectors are required
!      iright is TRUE if right eigenvectors are required

       ileft = .TRUE.
       iright = .TRUE.

!      Skip heading in data file

       READ (nin,*)
       READ (nin,*) n
       lda = n
       ldb = n
       ldvl = n
       ldvr = n
       lwork = 6*n
       ALLOCATE (a(lda,n),alphai(n),alphar(n),b(ldb,n),beta(n),lscale(n), &
          rscale(n),tau(n),vl(ldvl,ldvl),vr(ldvr,ldvr),work(lwork),select(n))

!      READ matrix A from data file
       READ (nin,*) (a(i,1:n),i=1,n)

!      READ matrix B from data file
       READ (nin,*) (b(i,1:n),i=1,n)

!      Balance matrix pair (A,B)
       job = 'B'
!      The NAG name equivalent of dggbal is f08whf
       CALL dggbal(job,n,a,lda,b,ldb,ilo,ihi,lscale,rscale,work,info)

!      Matrix A after balancing
!      ifail: behaviour on error exit
!             =0 for hard exit, =1 for quiet-soft, =-1 for noisy-soft
       ifail = 0
       CALL x04caf('General',' ',n,n,a,lda,'Matrix A after balancing',ifail)
       WRITE (nout,*)
       FLUSH (nout)

!      Matrix B after balancing
       ifail = 0
       CALL x04caf('General',' ',n,n,b,ldb,'Matrix B after balancing',ifail)
       WRITE (nout,*)
       FLUSH (nout)

!      Reduce B to triangular form using QR
       irows = ihi + 1 - ilo
       icols = n + 1 - ilo
!      The NAG name equivalent of dgeqrf is f08aef
       CALL dgeqrf(irows,icols,b(ilo,ilo),ldb,tau,work,lwork,info)

!      Apply the orthogonal transformation to matrix A
!      The NAG name equivalent of dormqr is f08agf
       CALL dormqr('L','T',irows,icols,irows,b(ilo,ilo),ldb,tau,a(ilo,ilo), &
          lda,work,lwork,info)

!      Initialize VL (if left eigenvectors are required)
       IF (ileft) THEN

          CALL f06qhf('General',n,n,zero,one,vl,ldvl)
          CALL f06qff('Lower',irows-1,irows-1,b(ilo+1,ilo),ldb,vl(ilo+1,ilo), &
             ldvl)

!         The NAG name equivalent of dorgqr is f08aff
          CALL dorgqr(irows,irows,irows,vl(ilo,ilo),ldvl,tau,work,lwork,info)
       END IF

!      Initialize VR (if right eigenvectors are required)
       IF (iright) CALL f06qhf('General',n,n,zero,one,vr,ldvr)

!      Compute the generalized Hessenberg form of (A,B)
       compq = 'V'
       compz = 'V'
!      The NAG name equivalent of dgghrd is f08wef
       CALL dgghrd(compq,compz,n,ilo,ihi,a,lda,b,ldb,vl,ldvl,vr,ldvr,info)

!      Matrix A in generalized Hessenberg form
       ifail = 0
       CALL x04caf('General',' ',n,n,a,lda,'Matrix A in Hessenberg form', &
          ifail)
       WRITE (nout,*)
       FLUSH (nout)

!      Matrix B in generalized Hessenberg form
       ifail = 0
       CALL x04caf('General',' ',n,n,b,ldb,'Matrix B in Hessenberg form', &
          ifail)

!      Routine DHGEQZ
!      Workspace query: jwork = -1

       jwork = -1
       job = 'S'

!      The NAG name equivalent of dhgeqz is f08xef
       CALL dhgeqz(job,compq,compz,n,ilo,ihi,a,lda,b,ldb,alphar,alphai,beta, &
          vl,ldvl,vr,ldvr,work,jwork,info)

       WRITE (nout,*)
       WRITE (nout,99999) nint(work(1))
       WRITE (nout,99998) lwork
       WRITE (nout,*)
       WRITE (nout,99997)
       WRITE (nout,99996)

!      Compute the generalized Schur form
!      if the workspace lwork is adequate
!      The Schur form also gives parameters
!      required to compute generalized eigenvalues

       IF (nint(work(1))<=lwork) THEN

!         The NAG name equivalent of dhgeqz is f08xef
          CALL dhgeqz(job,compq,compz,n,ilo,ihi,a,lda,b,ldb,alphar,alphai, &
             beta,vl,ldvl,vr,ldvr,work,lwork,info)

!         Print the generalized eigenvalues

          DO i = 1, n
             IF (beta(i)/=0.0E0_nag_wp) THEN
                WRITE (nout,99995) i, '(', alphar(i)/beta(i), ',', &
                   alphai(i)/beta(i), ')'
             ELSE
                WRITE (nout,99996) i
             END IF
          END DO
          WRITE (nout,*)
          FLUSH (nout)

!         Compute left and right generalized eigenvectors
!         of the balanced matrix

          howmny = 'B'
          IF (ileft .AND. iright) THEN
             side = 'B'
          ELSE IF (ileft) THEN
             side = 'L'
          ELSE IF (iright) THEN
             side = 'R'
          END IF

!         The NAG name equivalent of dtgevc is f08ykf
          CALL dtgevc(side,howmny,select,n,a,lda,b,ldb,vl,ldvl,vr,ldvr,n,m, &
             work,info)

          IF (iright) THEN

!            Compute right eigenvectors of the original matrix

             job = 'B'
             side = 'R'

!            The NAG name equivalent of dggbak is f08wjf
             CALL dggbak(job,side,n,ilo,ihi,lscale,rscale,n,vr,ldvr,info)

!            Normalize the right eigenvectors
             DO i = 1, n
                vr(1:n,i) = vr(1:n,i)/vr(1,i)
             END DO

!            Print the right eigenvectors

             ifail = 0
             CALL x04caf('General',' ',n,n,vr,ldvr, &
                'Right eigenvectors            ',ifail)

             WRITE (nout,*)
             FLUSH (nout)
          END IF

!         Compute left eigenvectors of the original matrix

          IF (ileft) THEN
             job = 'B'
             side = 'L'

!            The NAG name equivalent of dggbak is f08wjf
             CALL dggbak(job,side,n,ilo,ihi,lscale,rscale,n,vl,ldvl,info)

!            Normalize the left eigenvectors
             DO i = 1, n
                vl(1:n,i) = vl(1:n,i)/vl(1,i)
             END DO

!            Print the left eigenvectors

             ifail = 0
             CALL x04caf('General',' ',n,n,vl,ldvl,'Left eigenvectors',ifail)

          END IF
       ELSE
          WRITE (nout,99994)
       END IF

99999  FORMAT (1X,'Minimal required LWORK = ',I6)
99998  FORMAT (1X,'Actual value of  LWORK = ',I6)
99997  FORMAT (1X,'Generalized eigenvalues')
99996  FORMAT (1X,I4,5X,'Infinite eigenvalue')
99995  FORMAT (1X,I4,5X,A,F7.3,A,F7.3,A)
99994  FORMAT (1X,'Insufficient workspace for array WORK'/' in F08XEF/', &
          'DHGEQZ')
    END PROGRAM f08ykfe
